\input texinfo.tex  @c -*-texinfo-*-
@c %**start of header
@setfilename fd2inline.info
@settitle FD2InLine User's Guide
@set lastupdate 14 July 1996
@include version.texi
@c %**end of header

@ifinfo
This is a user's guide to FD2Inline @value{version}, a parser
converting FD files to GCC ``inlines''.

Copyright @copyright{} 1996 Kamil Iskra

Permission is granted to make and distribute verbatim copies of this
manual provided the copyright notice and this permission notice are
preserved on all copies.
@end ifinfo

@titlepage
@sp 10
@center @titlefont{FD2InLine}
@subtitle User's Guide to FD2InLine parser
@subtitle @value{lastupdate}
@author Kamil Iskra
@page
@end titlepage

@ifinfo
@node Top, Introduction, (dir), (dir)
This is a user's guide to @samp{FD2Inline} @value{version}, a parser
that converts Amiga shared libraries' FD files to format accepted by
GNU CC.

See file @file{COPYING} for GNU General Public License.

Last updated @value{lastupdate}.
@end ifinfo

@menu
* Introduction::  What is this program for?
* Installation::  How to install it?
* Usage::         How to use @samp{inlines} and @samp{FD2InLine}?
* Rebuilding::    How to recompile it?
* Internals::     How do @samp{inlines} work?
* Authors::       Or should it be called ``History''?

* Index::         Concept index.
@end menu

@node Introduction, Installation, Top, Top
@chapter Introduction
@cindex Introduction
@cindex What FD2InLine is

@samp{FD2InLine} is useful if you want to use @samp{GCC} for
Amiga-specific development and would like to make very efficient calls
to Amiga shared libraries' functions.

Format of calls to Amiga shared libraries' functions differs
substantially from the default format of C compilers
(@pxref{Background}). Therefore, some tricks are necessary if you want
to use these functions.

@samp{FD2InLine} is a parser that converts @samp{fd} files and
@samp{clib} files to @samp{GCC inlines}.

@samp{fd} and @samp{clib} files contain information about functions in
shared libraries (@pxref{Background}).

@samp{FD2InLine} reads these two files and outputs a file containing
information from input files merged in a special format, suitable to
use with @samp{GCC} compiler.

This output file contains so-called @dfn{inlines} --- one for each
function entry. Thanks to them @samp{GCC} can produce very efficient
code for Amiga shared libraries' functions calls.

@quotation
@emph{Note:} the term @samp{inlines} is misleading ---
@samp{FD2InLine} does not use @code{__inline} feature of @samp{GCC}
any longer (@pxref{New format}).
@end quotation

@node Installation, Usage, Introduction, Top
@chapter Installation
@cindex Installation
@cindex Where to put it

I assume you have @file{fd2inline-@value{version}-bin.lha} archive.

If you use recent release of @samp{GCC}, you might not need to install
anything. Starting with @samp{GCC} 2.7.2, new format (@pxref{New
format}) of @samp{inlines} should be available with the compiler.
However, the separate @file{fd2inline-@value{version}-bin.lha} archive
will always contain the latest version of @samp{FD2InLine} and
@samp{inlines}, which might not be the truth in case of @samp{ADE} or
@samp{Aminet} distributions.

Installation is very easy, so I didn't bother writing an Installer
script :-).

If you have an older version of @samp{inlines} installed, please
remove it now, or you might later have problems. Typically, you'll
have to remove the following subdirectories of @file{os-include}
directory: @file{inline}, @file{pragmas} and @file{proto}.

Next, please change current directory to @file{ADE:} and simply
extract @file{fd2inline-@value{version}-bin.lha} archive. This should
install everything in right places. More precisely, @samp{inlines} of
standard system libraries and devices will go to @file{include/inline}
directory and @samp{inlines} of 3rd party libraries will go to
@file{local/include/inline} directory. @file{fd2inline} executable
will go to @file{bin} directory and AmigaGuide documentation to
@file{guide}.

@node Usage, Rebuilding, Installation, Top
@chapter Usage
@cindex Usage

This chapter describes two aspects of using @samp{FD2InLine}:

@menu
* Using inlines::    Making efficient function calls
* Using fd2inline::  Creating @samp{inlines}
@end menu

@node Using inlines, Using fd2inline, Usage, Usage
@section Using inlines
@cindex Using inlines
@cindex Making efficient calls

Using @samp{inlines} is very simple. If you want to use a library
called @samp{foo.library} (or a device called @samp{bar.device}), just
include file @samp{<proto/foo.h>} (@samp{<proto/bar.h>}) and that's
it. For example:

@example
#include <proto/dos.h>

int main(void)
@{
   Delay(100); /* Wait for 2 seconds */
@}
@end example

Please @emph{always} include @file{proto} files, @emph{not}
@file{inline} files - @file{proto} files often fix some
incompatibilities between system headers and @samp{GCC}. Besides, this
technique makes your code more portable across various Amiga
compilers.

@cindex Preprocessor symbols
There are a few preprocessor symbols which alter the behaviour of
@samp{proto} and @samp{inlines} files:

@table @code
@cindex __NOLIBBASE__
@item __NOLIBBASE__
By default, @samp{proto} files make an external declaration of library
base pointer. You can disable this behaviour by defining
@code{__NOLIBBASE__} before including a @samp{proto} file.

@cindex __CONSTLIBBASEDECL__
@item __CONSTLIBBASEDECL__
The external declarations described above declare plain pointer
variables. The disadvantage of this is that library base variable has
to be reloaded every time some function is called. If you redefine
@code{__CONSTLIBBASEDECL__} to @code{const}, less reloading will be
necessary, so better code will be produced. However, declaring a
variable as @code{const} makes altering it impossible, so some dirty
hacks are necessary (like defining variable as plain in one file and
altering it only there. This @emph{won't} work with base relative
code, though).

@cindex <library>_BASE_NAME
@item <library>_BASE_NAME
Function definitions in @samp{inline} files refer to library base
variable through @code{<library>_BASE_NAME} symbol
(@code{AMIGAGUIDE_BASE_NAME} for @samp{amigaguide.library}, for
example). On top of @samp{inlines} file, this symbol is redefined to
appropriate library base variable name (@code{AmigaGuideBase}, for
example), @emph{unless} it's already defined. This way, you can make
inlines use a field of a structure as a library base, for example.

@item NO_INLINE_STDARG
This symbol prevents defining of inline macros for varargs functions
(@pxref{Old format}).

@cindex _USEOLDEXEC_
@item _USEOLDEXEC_
This one is used in @file{proto/exec.h} only. Unlike in @samp{SAS/C},
@file{proto/exec.h} uses @code{SysBase} variable as @samp{Exec}
library base by default. This is usually faster than direct
dereferencing of @code{0x00000004} (@pxref{Background}), since it
doesn't require reading from @samp{CHIP} memory (things might be even
worse if you use @samp{Enforcer} or @samp{CyberGuard}, which protect
low memory region). However, in some low-level cases (like startup
code) you might prefer dereferencing @code{0x00000004}. To do this,
define @code{_USEOLDEXEC_} before including @file{proto/exec.h}.
@end table

@node Using fd2inline, , Using inlines, Usage
@section Using fd2inline
@cindex Using FD2Inline
@cindex Creating inlines

You invoke @samp{FD2InLine} by writing:

@example
@file{fd2inline} [options] @var{fd-file} @var{clib-file} [[-o] @var{output-file}]
@end example

Variables have the following meaning:

@table @var
@item fd-file
A file name of an input @samp{fd} file.

@item clib-file
A file name of an input @samp{clib} file.

@item output-file
A file name of an output @samp{inlines} file. If it is not specified
(or if @samp{-} is specified), standard output will be used instead.
The file name can be preceded with a @samp{-o}, for compatibility with
most UN*X software.
@end table

The following options can be specified (anywhere on the command line):

@table @samp
@item --new
Produce new format @samp{inlines}.

@item --old
Produce old format @samp{inlines}.

@item --stubs
Produce library stubs.
@end table

@xref{Internals}, for more information.

Example:

@example
fd2inline FD:exec_lib.fd ADE:os-include/clib/exec_protos.h -o ADE:include/inline/exec.h
@end example

This will build file @file{exec.h} containing new format
@samp{inlines} of @samp{exec.library} in @file{ADE:include/inline}
directory.

If you want to add support for @samp{GCC} to a library, creating
@samp{inlines} is not enough --- you should also create @samp{proto}
file.

Use one of existing files as a template. Some libraries (like for
example @samp{dos.library}) have broken header files and @samp{GCC}
generates warning messages. In order to avoid them, you have to
include various headers in @samp{proto} file before including
@samp{clib} file.

You might also want to create @samp{pragmas} file, which might be
necessary for badly written @samp{SAS/C} sources. @file{pragmas} are
generated automatically during building of @samp{FD2InLine} by an
@samp{AWK} script, so you might either have a look at
@file{fd2inline-@value{version}-src.lha} archive or simply create
@file{pragmas} by hand.

Creating a linker library with stubs might also be useful, in case
somebody didn't want to, or couldn't, use inline headers.

@file{fd2inline-@value{version}-src.lha} contains necessary support
for this. For example, to generate @file{libexec.a} library with
@samp{exec.library} stubs, you'd have to type:

@example
make alllib INCBASE=exec LIBBASE=exec
@end example

This would create two @file{libexec.a} libraries: plain and base
relative one. Of course, this particular example doesn't make much
sense since @file{libamiga.a} already contains this stubs.

@code{INCBASE} and @code{LIBBASE} specify basenames of (input)
@samp{proto} and @samp{fd} files and (output) library. This will often
be the same, but not always. For example, in case of @samp{MUI}
@code{INCBASE} has to be set to @code{muimaster}, but @code{LIBBASE}
might be set to @code{mui}.

@node Rebuilding, Internals, Usage, Top
@chapter Rebuilding
@cindex Rebuilding

First, you have to download @file{fd2inline-@value{version}-src.lha}
archive.

Unarchive it. You might either build @samp{FD2InLine} in source
archive or in a separate, build archive. I suggest the latter. Type:

@example
lha -mraxe x fd2inline-@value{version}-src.lha
mkdir fd2inline-bin
cd fd2inline-bin
sh ../fd2inline-@value{version}/configure --prefix=/ade
make
@end example

This should build @samp{FD2InLine} executable and @samp{inlines}.

Please note that @samp{fd} files should be available in directory
@file{ADE:os-lib/fd}. If you store them in some other place, you'll
have to edit @file{Makefile} before invoking @code{make} (variable
@code{FD_DIR}).

You can then type:

@example
make install
@end example

This will install new @samp{fd2inline}, @samp{inlines} and
documentation in appropriate subdirectories of @file{ADE:}.

@file{fd2inline-@value{version}-src.lha} archive contains three
patches in context diff format. They fix bugs in OS 3.1 headers and
@samp{fd} files. Without applying @file{amigaguide_lib.fd.diff} you
won't be able to build @samp{inlines} for @samp{amigaguide.library}.
Two other patches rename an argument @code{true} to @code{tf}, since
@code{true} is a reserved word in @samp{C++}. Use @code{patch} to
apply the patches, for example:

@example
cd ADE:os-lib/fd
patch <amigaguide_lib.fd.diff
@end example

@cindex Source code
A few words about the source code:

I know, it's not state-of-the-art @samp{C} programming example.
However, believe me, it was in @emph{much} worse condition when I took
it over. In its current state it is at least readable (if you use tab
size 3, as I do :-). I think that rewriting it in @samp{C++} would
clean it up considerably (it's already written in @samp{OO} fashion,
so this should be quite easy). Using @code{flex} and @code{bison} to
create the parser would also be a nice thing, I guess. However, I
don't think it's worth the effort. But, if somebody wanted to do it:
feel free, I'll be more than happy.

@node Internals, Authors, Rebuilding, Top
@chapter Internals
@cindex Internals

This chapter describes the implementation details of @samp{inlines}.

@menu
* Background::    Function calls in shared libraries.
* Old format::    Inlines that used @code{__inline}
* New format::    Inlines that use preprocessor.
* Stubs format::  Not really inlines, but...
@end menu

@node Background, Old format, Internals, Internals
@section Background
@cindex Background
@cindex Function calls format in shared libraries

This section describes calling conventions used in Amiga shared
libraries.

@cindex Libraries
User-callable functions in AmigaOS are organized in @dfn{libraries}.

@cindex Library base
@cindex Jump table
From our point of view, most important part of a library is a
@dfn{library base}. It always resides in RAM and contains library
variables and a @dfn{jump table}. The location of library base varies.
You can obtain the library base location of main system library ---
@code{exec.library} --- by dereferencing @code{0x00000004}. Locations
of other libraries' bases can be obtained using @code{OpenLibrary}
function of @samp{exec.library}.

Without providing unnecessary details, every function has a fixed
place in library's jump table. In order to call a function, one has to
jump into this place.

@cindex Function arguments
Most functions require some arguments. In @samp{C}, these are usually
passed on CPU stack. However, for some obscure reason, AmigaOS system
designers decided that arguments to shared libraries should be passed
in CPU registers.

@cindex FD files
All the information required to make function calls are provided in
@dfn{fd} files. Every shared library should have such a file. It
provides information about the name a library base variable should
have and, for every function separately, about offset in jump table
where the function resides and about registers in which arguments
should be passed.

@cindex CLIB files
In order to check if arguments passed to function have correct type, C
compiler needs function prototypes. These are provided in @dfn{clib}
files --- every library should have such a file.

@cindex Varargs functions
Starting with AmigaOS release 2.0, some functions have been provided
which accept variable number of arguments (so-called @dfn{varargs
functions}). Actually, these are only @samp{C} language stubs.
Internally, all optional arguments have to be put into an array of
@samp{long ints} and address of this array is passed to fixed args
library function.

@cindex Linker libraries
To implement calls to shared libraries' functions, compiler vendors
have to either use some compiler-dependent tricks to make this calls
directly (so-called @dfn{in line}), or provide linker libraries with
functions stubs, usually written in assembler. In the latter case,
function call from user's code is compiled as usual --- arguments are
passed on stack. Then, in linking stage, a library stub gets linked
in, and when it's called, it moves arguments from stack to appropriate
registers and jumps to library jump table. Needless to say, this is
slower than making a call in line.

@node Old format, New format, Background, Internals
@section Old format
@cindex Old inline format

@example
extern __inline APTR
OpenAmigaGuideA(BASE_PAR_DECL struct NewAmigaGuide *nag, struct TagItem *attrs)
@{
   BASE_EXT_DECL
   register APTR res __asm("d0");
   register struct Library *a6 __asm("a6") = BASE_NAME;
   register struct NewAmigaGuide *a0 __asm("a0") = nag;
   register struct TagItem *a1 __asm("a1") = attrs;
   __asm volatile ("jsr a6@@(-0x36:W)"
   : "=r" (res)
   : "r" (a6), "r" (a0), "r" (a1)
   : "d0", "d1", "a0", "a1", "cc", "memory");
   return res;
@}
@end example

In this implementation, Amiga shared libraries' function stubs are
external functions. They are defined as @code{__inline}, what makes
@samp{GCC} insert them at every place of call. Mysterious
@code{BASE_PAR_DECL} and @code{BASE_EXT_DECL} defines are hacks
necessary for local library base support (which is quite hard to
achieve, so I won't describe it here). The biggest disadvantage of
this @samp{inlines} is the fact that @samp{GCC} becomes very slow and
requires huge amount of memory when compiling them. Besides, inlining
works with optimization enabled only.

@example
#ifndef NO_INLINE_STDARG
#define OpenAmigaGuide(a0, tags...) \
   (@{ULONG _tags[] = @{ tags @}; OpenAmigaGuideA((a0), (struct TagItem *)_tags);@})
#endif /* !NO_INLINE_STDARG */
@end example

@cindex Varargs problems
This is how @samp{varargs} functions are implemented. Handling them
cannot be made using @code{__inline} functions, since @code{__inline}
functions require fixed number of arguments. Therefore, unique
features of GCC preprocessor (such as @samp{varargs macros}) have to
be used, instead. This has some drawbacks, unfortunately. Since this
are actually preprocessor macros and not function calls, you can't
make tricky things that involve preprocessor inside them. For example:

@example
#include <proto/amigaguide.h>

#define OPENAG_BEG OpenAmigaGuide(
#define OPENAG_END , TAG_DONE)

void f(void)
@{
   OPENAG_BEG "a_file.guide" OPENAG_END;
   OpenAmigaGuide(
#ifdef ABC
      "abc.guide",
#else
      "def.guide",
#endif
      TAG_DONE);
@}
@end example

Neither of @code{OpenAmigaGuide()} calls above is handled correctly.

In case of the first call, you get an error:

@example
unterminated macro call
@end example

By the time preprocessor attempts to unwind @code{OpenAmigaGuide}
macro, @code{OPENAG_END} is not yet unwound, so preprocessor can't
find closing bracket. This code might look artificial, but @samp{MUI}
for example defines such macros to make code look more pretty.

In case of the second call, you'll see:

@example
warning: preprocessing directive not recognized within macro arg
@end example

A workaround would be to either surround whole function calls with
conditions, or to conditionally define a preprocessor symbol
@code{GUIDE} somewhere above and simply put @code{GUIDE} as a function
argument:

@example
#ifdef ABC
#define GUIDE "abc.guide"
#else
#define GUIDE "def.guide"
#endif

void f(void)
@{
#ifdef ABC
   OpenAmigaGuide("abc.guide", TAG_DONE);
#else
   OpenAmigaGuide("def.guide", TAG_DONE);
#endif
   OpenAmigaGuide(GUIDE, TAG_DONE);
@}
@end example

Because of this drawbacks, @samp{varargs inlines} can be disabled by
defining @code{NO_INLINE_STDARG} before including @samp{proto} file.
You'll need a library with function stubs in such case.

@node New format, Stubs format, Old format, Internals
@section New format
@cindex New inline format

@example
#define OpenAmigaGuideA(nag, attrs) \
   LP2(0x36, APTR, OpenAmigaGuideA, struct NewAmigaGuide *, nag, a0, struct TagItem *, attrs, a1, \
   , AMIGAGUIDE_BASE_NAME)
@end example

As you can see, this implementation is much more compact. @code{LP2}
macro (and others) are defined in @file{inline/macros.h}, which is
being included at the beginning of every @samp{inline} file.

@example
#define LP2(offs, rt, name, t1, v1, r1, t2, v2, r2, bt, bn) \
(@{							\
   t1 _##name##_v1 = (v1);				\
   t2 _##name##_v2 = (v2);				\
   @{							\
      register rt _##name##_re __asm("d0");		\
      register struct Library *const _##name##_bn __asm("a6") = (struct Library*)(bn);\
      register t1 _n1 __asm(#r1) = _##name##_v1;	\
      register t2 _n2 __asm(#r2) = _##name##_v2;	\
      __asm volatile ("jsr a6@@(-"#offs":W)"		\
      : "=r" (_##name##_re)				\
      : "r" (_##name##_bn), "r"(_n1), "r"(_n2)		\
      : "d0", "d1", "a0", "a1", "cc", "memory");	\
      _##name##_re;					\
   @}							\
@})
@end example

If you compare this with old inlines (@pxref{Old format}) you'll
notice many similarities. Indeed, both implementations use the same
tricks.

However, with new inlines, inlining is performed very early, at
preprocessing stage. This makes compilation much faster, less memory
hungry and independent of optimization options used. This also makes
it very easy to use local library bases --- all that's needed is
defining a local variable with the same name as library base.

Unfortunately, using preprocessor instead of compiler for making
function calls has its drawback, as described earlier (@pxref{Old
format}). There is nothing you can do about it but modify your code,
I'm afraid.

Depending on the type of function, @samp{fd2inline} generates calls to
different @samp{LP} macros.

Macros are distinguished by one or more of qualifiers described below:

@table @samp
@item digit
As you might have already guessed, digit indicates the number of
arguments a function accepts. Therefore, it's mandatory.

@item NR
This indicates ``no return'' (@code{void}) function.

@item A4, A5
These two are used when one of the arguments has to be in either
@samp{a4} or @samp{a5} register. In certain situations, these
registers have special meaning and have to be handled more carefully.

@item UB
This indicates ``user base'' --- library base pointer has to be
specified explicitly by user. Currently, this is used for
@samp{cia.resource} only. Since there are two @samp{CIA} chips,
programmer has to specify which one [s]he wants to use.

@item FP
This means that one of the arguments has a ``pointer to function''
type. To overcome strange @samp{C} syntax rules in this case, inside
@samp{FP} macros a @code{typedef} to @code{__fpt} is performed.
@samp{inline} file passes @code{__fpt} as argument's type to @samp{LP}
macro. The actual type of the argument, in a form suitable for a
@code{typedef}, is passed as an additional, last argument.

@end table

As you can see, there could be more than a hundred different
variations of @samp{LP} macros. @file{inline/macros.h} contains only
34, which are used in current OS version and supported 3rd party
libraries. More macros will be added in future, if needed.

If you had a careful look at the definition of @code{OpenAmigaGuideA}
on the beginning of this section, you might have noticed that next to
last argument to @samp{LP} macro is not used. New inlines were not
implemented in one evening and they came through many modifications.
This unused argument (which was once a type of library base pointer)
is provided for backwards compatibility. Actually, there are more
unnecessary arguments, like function and argument names, but we
decided to left them in peace.

@node Stubs format, , New format, Internals
@section Stubs format
@cindex Stubs inline format

Stubs format is very similar to old format (@pxref{Old format}). The
functions are not defined as @code{extern}, however.

The main difference is format of @samp{varargs} functions --- they are
actual functions, not preprocessor macros.

@example
APTR OpenAmigaGuide(struct NewAmigaGuide *nag, int tag, ...)
@{
   return OpenAmigaGuideA(nag, (struct TagItem *)&tag);
@}
@end example

This format is not suitable for inlining, and it is not provided for
this purpose. It is provided for building of linker libraries with
stubs (@pxref{Using fd2inline}).

@node Authors, Index, Internals, Top
@chapter Authors
@cindex Authors
@cindex Other parsers
@cindex History
@cindex Reporting bugs
@cindex Address
@cindex Latest version

First parser for @samp{GCC inlines} was written in @samp{Perl} by
Markus Wild.

It had several limitations, which were apparently hard to fix in
@samp{Perl}. That's why Wolfgang Baron decided to write a new parser
in @samp{C}.

However, for some reason he has never finished it. In early 1995
Rainer F. Trunz took over its development and ``improved, updated,
simply made it workable'' (it's a quote from changes log). It still
contained quite a lot of bugs, though.

In more-or-less the same time I started a discussion on
@samp{amiga-gcc-port} mailing list about improving quality of
@samp{inlines}. The most important idea came from Matthias Fleischer
--- he introduced new format of @samp{inlines} (@pxref{New format}).
Since I started the discussion, I volunteered to make improvements to
@samp{inlines} parser. Having no idea about programming in
@samp{Perl}, I decided to modify the parser written in @samp{C}. I
fixed all the bugs known to me, added some new features and wrote this
terribly long documentation :-).

Not all of the files distributed in @samp{FD2InLine} archives were
created by me or @samp{FD2InLine}. Most of the files in
@file{include/proto}, as well as a few files in @file{include/inline}
(@file{alib.h}, @file{strsub.h} and @file{stubs.h}) were written by
Gunther Nikl (with some modifications by Joerg Hoehle).

If you have any comments concerning this work, please write to:

ade-gcc@@ninemoons.com

This is a list where most of ADE GCC developers and activists are
subscribed, so you are practically guaranteed to get a reply.

However, if you for some reason wanted to contact me personally, do it
in one of the following ways:

@itemize @bullet
@item
E-mail (preferred :-):

iskra@@student.uci.agh.edu.pl

Should be valid until October 1999 (at least I hope so :-).

@item
Snail-mail (expect to wait long for a reply :-):

@format
Kamil Iskra
Luzycka 51/258
30-658 Krakow
Poland
@end format
@end itemize

Latest version of this package should always be available on my WWW
page:

http://student.uci.agh.edu.pl/~iskra

@node Index, , Authors, Top
@chapter Index
@printindex cp

@contents
@bye
